=head1 PURPOSE

Check that certain nonsense fixed variables cause compilation errors.

=head1 AUTHOR

Toby Inkster E<lt>tobyink@cpan.orgE<gt>.

=head1 COPYRIGHT AND LICENCE

This software is copyright (c) 2013 by Toby Inkster.

This is free software; you can redistribute it and/or modify it under
the same terms as the Perl 5 programming language system itself.

=cut

use strict;
use warnings;
use Test::More tests => 4;

ok !eval q{ use Fixed; fix @a; 1 }, chomp($@) && $@;
ok !eval q{ use Fixed; fix %a; 1 }, chomp($@) && $@;
ok !eval q{ use Fixed; fix *a; 1 }, chomp($@) && $@;
ok !eval q{ use Fixed; fix  a; 1 }, chomp($@) && $@;

